@@ -16,14 +16,14 @@
 
 set -eux -o pipefail
 
-if (($# < 2)); then
-  echo "ERROR: given ! ($#) number of parameters but expect 2."
-  echo "USAGE: ./check_install_golang.sh VERSION_TO_INSTALL INSTALL_PATH"
+if (($# < 1)); then
+  echo "ERROR: given ! ($#) parameters but expected 1."
+  echo "USAGE: ./check_install_golang.sh INSTALL_PATH"
   exit 1
 fi
 
-VERSION_TO_INSTALL=${1}
-INSTALL_PATH=${2}
+VERSION_TO_INSTALL=$(grep '^GO_VERSION' Makefile | awk '{ print $3 }')
+INSTALL_PATH=${1}
 
 function current_arch() {
   case $(arch) in
@@ -51,7 +51,7 @@ function check_and_install_golang() {
   fi
 
   # golang has been installed and check its version
-  if [[ $(go version) =~ (([0-9]+)\.([0-9]+).([0-9]+).([\.0-9]*)) ]]; then
+  if [[ $(go version | cut -d' ' -f 3) =~ go(([0-9]+)\.([0-9]+).([0-9]+)*) ]]; then
     HOST_VERSION=${BASH_REMATCH[1]}
     if [ $HOST_VERSION = $VERSION_TO_INSTALL ]; then
       echo "go version on the host looks good : $HOST_VERSION"
@@ -67,20 +67,28 @@ function check_and_install_golang() {
 
 # install_golang takes two parameters version and path to install.
 function install_golang() {
-  echo "Installing golang version: $1 on $2"
-  pushd /tmp >/dev/null
+  local -r GO_VER="$1"
+  local -r GO_DIR="$2/go"
+  echo "Installing golang version: $GO_VER in $GO_DIR"
 
   INSTALLOS=linux
   if [[ "$OSTYPE" == "darwin"* ]]; then
     INSTALLOS=darwin
   fi
+
+  local -r GO_TGZ="go${GO_VER}.${INSTALLOS}-${ARCH}.tar.gz"
+  pushd /tmp
+
   # using sudo because previously installed versions might have been installed by a different user.
   # as it was the case on jenkins VM.
-  sudo curl -qL -O "https://storage.googleapis.com/golang/go${1}.${INSTALLOS}-${ARCH}.tar.gz" &&
-    sudo tar -xzf go${1}.${INSTALLOS}-${ARCH}.tar.gz &&
-    sudo rm -rf "${2}/go" &&
-    sudo mv go "${2}/" && sudo chown -R $(whoami): ${2}/go
+  sudo rm -rf "$GO_TGZ"
+  curl -qL -O "https://storage.googleapis.com/golang/$GO_TGZ"
+  sudo rm -rf "$GO_DIR"
+  sudo mkdir -p "$GO_DIR"
+  sudo tar -C "$GO_DIR" --strip-components=1 -xzf "$GO_TGZ"
+
   popd >/dev/null
+  echo "installed in $GO_DIR: $($GO_DIR/bin/go version)"
 }
 
 check_and_install_golang
