@@ -72,7 +72,7 @@ func enableAddonGCPAuth(cfg *config.ClusterConfig) error {
 	// Grab credentials from where GCP would normally look
 	ctx := context.Background()
 	creds, err := google.FindDefaultCredentials(ctx)
-	if err != nil || creds.JSON == nil {
+	if err != nil {
 		if detect.IsCloudShell() {
 			if c := os.Getenv("CLOUDSDK_CONFIG"); c != "" {
 				f, err := ioutil.ReadFile(path.Join(c, "application_default_credentials.json"))
@@ -98,6 +98,11 @@ func enableAddonGCPAuth(cfg *config.ClusterConfig) error {
 		return nil
 	}
 
+	if creds.JSON == nil {
+		out.WarningT("You have authenticated with a service account that does not have an associated JSON. The GCP Auth requires credentials with a JSON file to in order to continue. The image pull secret has been imported.")
+		return nil
+	}
+
 	// Actually copy the creds over
 	f := assets.NewMemoryAssetTarget(creds.JSON, credentialsPath, "0444")
 
@@ -164,19 +169,20 @@ func createPullSecret(cc *config.ClusterConfig, creds *google.Credentials) error
 		}
 
 		for _, n := range namespaces.Items {
+			if skipNamespace(n.Name) {
+				continue
+			}
 			secrets := client.Secrets(n.Name)
 
 			exists := false
-			if !Refresh {
-				secList, err := secrets.List(context.TODO(), metav1.ListOptions{})
-				if err != nil {
-					return err
-				}
-				for _, s := range secList.Items {
-					if s.Name == secretName {
-						exists = true
-						break
-					}
+			secList, err := secrets.List(context.TODO(), metav1.ListOptions{})
+			if err != nil {
+				return err
+			}
+			for _, s := range secList.Items {
+				if s.Name == secretName {
+					exists = true
+					break
 				}
 			}
 
@@ -253,7 +259,7 @@ func refreshExistingPods(cc *config.ClusterConfig) error {
 	}
 	for _, n := range namespaces.Items {
 		// Ignore kube-system and gcp-auth namespaces
-		if n.Name == metav1.NamespaceSystem || n.Name == namespaceName {
+		if skipNamespace(n.Name) {
 			continue
 		}
 
@@ -323,11 +329,33 @@ func disableAddonGCPAuth(cfg *config.ClusterConfig) error {
 
 	// No need to check for an error here, if the secret doesn't exist, no harm done.
 	for _, n := range namespaces.Items {
+		if skipNamespace(n.Name) {
+			continue
+		}
 		secrets := client.Secrets(n.Name)
 		err := secrets.Delete(context.TODO(), secretName, metav1.DeleteOptions{})
 		if err != nil {
 			klog.Infof("error deleting secret: %v", err)
 		}
+
+		serviceaccounts := client.ServiceAccounts(n.Name)
+		salist, err := serviceaccounts.List(context.TODO(), metav1.ListOptions{})
+		if err != nil {
+			klog.Infof("error getting service accounts: %v", err)
+			return err
+		}
+		for _, sa := range salist.Items {
+			for i, ps := range sa.ImagePullSecrets {
+				if ps.Name == secretName {
+					sa.ImagePullSecrets = append(sa.ImagePullSecrets[:i], sa.ImagePullSecrets[i+1:]...)
+					_, err := serviceaccounts.Update(context.TODO(), &sa, metav1.UpdateOptions{})
+					if err != nil {
+						return err
+					}
+					break
+				}
+			}
+		}
 	}
 
 	return nil
@@ -367,3 +395,7 @@ func verifyGCPAuthAddon(cc *config.ClusterConfig, name string, val string) error
 
 	return err
 }
+
+func skipNamespace(name string) bool {
+	return name == metav1.NamespaceSystem || name == namespaceName
+}
