@@ -69,6 +69,12 @@ oom_score = 0
 [cgroup]
   path = ""
 
+[proxy_plugins]
+# fuse-overlayfs is used for rootless
+[proxy_plugins."fuse-overlayfs"]
+  type = "snapshot"
+  address = "/run/containerd-fuse-overlayfs.sock"
+
 [plugins]
   [plugins.cgroups]
     no_prometheus = false
@@ -80,6 +86,7 @@ oom_score = 0
     stats_collect_period = 10
     enable_tls_streaming = false
     max_container_log_line_size = 16384
+    restrict_oom_score_adj = {{ .RestrictOOMScoreAdj }}
 
 	[plugins."io.containerd.grpc.v1.cri"]
       [plugins."io.containerd.grpc.v1.cri".containerd]
@@ -90,7 +97,7 @@ oom_score = 0
               SystemdCgroup = {{ .SystemdCgroup }}
 
     [plugins.cri.containerd]
-      snapshotter = "overlayfs"
+      snapshotter = "{{ .Snapshotter }}"
       [plugins.cri.containerd.default_runtime]
         runtime_type = "io.containerd.runc.v2"
       [plugins.cri.containerd.untrusted_workload_runtime]
@@ -140,6 +147,22 @@ func (r *Containerd) Style() style.Enum {
 	return style.Containerd
 }
 
+// parseContainerdVersion parses version from containerd --version
+func parseContainerdVersion(line string) (string, error) {
+	// containerd github.com/containerd/containerd v1.0.0 89623f28b87a6004d4b785663257362d1658a729
+	words := strings.Split(line, " ")
+	if len(words) >= 4 && words[0] == "containerd" {
+		version := strings.Replace(words[2], "v", "", 1)
+		version = strings.SplitN(version, "~", 2)[0]
+		if _, err := semver.Parse(version); err != nil {
+			parts := strings.SplitN(version, "-", 2)
+			return parts[0], nil
+		}
+		return version, nil
+	}
+	return "", fmt.Errorf("unknown version: %q", line)
+}
+
 // Version retrieves the current version of this runtime
 func (r *Containerd) Version() (string, error) {
 	c := exec.Command("containerd", "--version")
@@ -147,12 +170,11 @@ func (r *Containerd) Version() (string, error) {
 	if err != nil {
 		return "", errors.Wrapf(err, "containerd check version.")
 	}
-	// containerd github.com/containerd/containerd v1.2.0 c4446665cb9c30056f4998ed953e6d4ff22c7c39
-	words := strings.Split(rr.Stdout.String(), " ")
-	if len(words) >= 4 && words[0] == "containerd" {
-		return strings.Replace(words[2], "v", "", 1), nil
+	version, err := parseContainerdVersion(rr.Stdout.String())
+	if err != nil {
+		return "", err
 	}
-	return "", fmt.Errorf("unknown version: %q", rr.Stdout.String())
+	return version, nil
 }
 
 // SocketPath returns the path to the socket file for containerd
@@ -178,23 +200,31 @@ func (r *Containerd) Available() error {
 }
 
 // generateContainerdConfig sets up /etc/containerd/config.toml
-func generateContainerdConfig(cr CommandRunner, imageRepository string, kv semver.Version, forceSystemd bool, insecureRegistry []string) error {
+func generateContainerdConfig(cr CommandRunner, imageRepository string, kv semver.Version, forceSystemd bool, insecureRegistry []string, inUserNamespace bool) error {
 	cPath := containerdConfigFile
 	t, err := template.New("containerd.config.toml").Parse(containerdConfigTemplate)
 	if err != nil {
 		return err
 	}
 	pauseImage := images.Pause(kv, imageRepository)
+	snapshotter := "overlayfs"
+	if inUserNamespace {
+		snapshotter = "fuse-overlayfs"
+	}
 	opts := struct {
 		PodInfraContainerImage string
 		SystemdCgroup          bool
 		InsecureRegistry       []string
 		CNIConfDir             string
+		RestrictOOMScoreAdj    bool
+		Snapshotter            string
 	}{
 		PodInfraContainerImage: pauseImage,
 		SystemdCgroup:          forceSystemd,
 		InsecureRegistry:       insecureRegistry,
 		CNIConfDir:             cni.ConfDir,
+		RestrictOOMScoreAdj:    inUserNamespace,
+		Snapshotter:            snapshotter,
 	}
 	var b bytes.Buffer
 	if err := t.Execute(&b, opts); err != nil {
@@ -208,7 +238,7 @@ func generateContainerdConfig(cr CommandRunner, imageRepository string, kv semve
 }
 
 // Enable idempotently enables containerd on a host
-func (r *Containerd) Enable(disOthers, forceSystemd bool) error {
+func (r *Containerd) Enable(disOthers, forceSystemd, inUserNamespace bool) error {
 	if disOthers {
 		if err := disableOthers(r, r.Runner); err != nil {
 			klog.Warningf("disableOthers: %v", err)
@@ -217,13 +247,19 @@ func (r *Containerd) Enable(disOthers, forceSystemd bool) error {
 	if err := populateCRIConfig(r.Runner, r.SocketPath()); err != nil {
 		return err
 	}
-	if err := generateContainerdConfig(r.Runner, r.ImageRepository, r.KubernetesVersion, forceSystemd, r.InsecureRegistry); err != nil {
+	if err := generateContainerdConfig(r.Runner, r.ImageRepository, r.KubernetesVersion, forceSystemd, r.InsecureRegistry, inUserNamespace); err != nil {
 		return err
 	}
 	if err := enableIPForwarding(r.Runner); err != nil {
 		return err
 	}
 
+	if inUserNamespace {
+		if err := r.Init.EnableNow("containerd-fuse-overlayfs"); err != nil {
+			return err
+		}
+	}
+
 	// Otherwise, containerd will fail API requests with 'Unimplemented'
 	return r.Init.Restart("containerd")
 }
@@ -233,10 +269,14 @@ func (r *Containerd) Disable() error {
 	return r.Init.ForceStop("containerd")
 }
 
-// ImageExists checks if an image exists, expected input format
+// ImageExists checks if image exists based on image name and optionally image sha
 func (r *Containerd) ImageExists(name string, sha string) bool {
-	c := exec.Command("/bin/bash", "-c", fmt.Sprintf("sudo ctr -n=k8s.io images check | grep %s | grep %s", name, sha))
-	if _, err := r.Runner.RunCmd(c); err != nil {
+	c := exec.Command("/bin/bash", "-c", fmt.Sprintf("sudo ctr -n=k8s.io images check | grep %s", name))
+	rr, err := r.Runner.RunCmd(c)
+	if err != nil {
+		return false
+	}
+	if sha != "" && !strings.Contains(rr.Output(), sha) {
 		return false
 	}
 	return true
@@ -290,6 +330,16 @@ func (r *Containerd) RemoveImage(name string) error {
 	return removeCRIImage(r.Runner, name)
 }
 
+// TagImage tags an image in this runtime
+func (r *Containerd) TagImage(source string, target string) error {
+	klog.Infof("Tagging image %s: %s", source, target)
+	c := exec.Command("sudo", "ctr", "-n=k8s.io", "images", "tag", source, target)
+	if _, err := r.Runner.RunCmd(c); err != nil {
+		return errors.Wrapf(err, "ctr images tag")
+	}
+	return nil
+}
+
 func gitClone(cr CommandRunner, src string) (string, error) {
 	// clone to a temporary directory
 	rr, err := cr.RunCmd(exec.Command("mktemp", "-d"))
@@ -343,6 +393,10 @@ func downloadRemote(cr CommandRunner, src string) (string, error) {
 
 // BuildImage builds an image into this runtime
 func (r *Containerd) BuildImage(src string, file string, tag string, push bool, env []string, opts []string) error {
+	if err := r.initBuildkitDaemon(); err != nil {
+		return fmt.Errorf("failed to init buildkit daemon: %v", err)
+	}
+
 	// download url if not already present
 	dir, err := downloadRemote(r.Runner, src)
 	if err != nil {
@@ -393,6 +447,34 @@ func (r *Containerd) BuildImage(src string, file string, tag string, push bool,
 	return nil
 }
 
+// PushImage pushes an image
+func (r *Containerd) PushImage(name string) error {
+	klog.Infof("Pushing image %s: %s", name)
+	c := exec.Command("sudo", "ctr", "-n=k8s.io", "images", "push", name)
+	if _, err := r.Runner.RunCmd(c); err != nil {
+		return errors.Wrapf(err, "ctr images push")
+	}
+	return nil
+}
+func (r *Containerd) initBuildkitDaemon() error {
+	// if daemon is already running, do nothing
+	cmd := exec.Command("pgrep", "buildkitd")
+	if _, err := r.Runner.RunCmd(cmd); err == nil {
+		return nil
+	}
+
+	// otherwise, start daemon
+	cmd = exec.Command("/bin/bash", "-c", "sudo -b buildkitd --oci-worker false --containerd-worker true --containerd-worker-namespace k8s.io &> /dev/null")
+	if _, err := r.Runner.RunCmd(cmd); err != nil {
+		return fmt.Errorf("failed to start buildkit daemon: %v", err)
+	}
+
+	// give the daemon time to finish starting up or image build will fail
+	time.Sleep(1 * time.Second)
+
+	return nil
+}
+
 // CGroupDriver returns cgroup driver ("cgroupfs" or "systemd")
 func (r *Containerd) CGroupDriver() (string, error) {
 	info, err := getCRIInfo(r.Runner)
