@@ -236,6 +236,10 @@ func (f *FakeRunner) Copy(assets.CopyableFile) error {
 	return nil
 }
 
+func (f *FakeRunner) CopyFrom(assets.CopyableFile) error {
+	return nil
+}
+
 func (f *FakeRunner) Remove(assets.CopyableFile) error {
 	return nil
 }
@@ -664,7 +668,7 @@ func TestEnable(t *testing.T) {
 			if err != nil {
 				t.Fatalf("New(%s): %v", tc.runtime, err)
 			}
-			err = cr.Enable(true, false)
+			err = cr.Enable(true, false, false)
 			if err != nil {
 				t.Errorf("%s disable unexpected error: %v", tc.runtime, err)
 			}
