@@ -33,9 +33,9 @@ import (
 	"github.com/Delta456/box-cli-maker/v2"
 	"github.com/briandowns/spinner"
 	"github.com/mattn/go-isatty"
+	"github.com/spf13/pflag"
 
 	"k8s.io/klog/v2"
-	"k8s.io/minikube/pkg/minikube/localpath"
 	"k8s.io/minikube/pkg/minikube/out/register"
 	"k8s.io/minikube/pkg/minikube/style"
 	"k8s.io/minikube/pkg/minikube/translate"
@@ -68,6 +68,8 @@ var (
 	JSON = false
 	// spin is spinner showed at starting minikube
 	spin = spinner.New(spinner.CharSets[style.SpinnerCharacter], 100*time.Millisecond)
+	// defaultBoxCfg is the default style config for cli box output
+	defaultBoxCfg = box.Config{Py: 1, Px: 4, Type: "Round", Color: "Red"}
 )
 
 // MaxLogEntries controls the number of log entries to show for each source
@@ -113,23 +115,33 @@ func Styled(st style.Enum, format string, a ...V) {
 	}
 }
 
-func boxedCommon(printFunc func(format string, a ...interface{}), format string, a ...V) {
-	box := box.New(box.Config{Py: 1, Px: 4, Type: "Round"})
-	if useColor {
-		box.Config.Color = "Red"
+func boxedCommon(printFunc func(format string, a ...interface{}), cfg box.Config, title string, format string, a ...V) {
+	box := box.New(cfg)
+	if !useColor {
+		box.Config.Color = nil
 	}
 	str := Sprintf(style.None, format, a...)
-	printFunc(box.String("", strings.TrimSpace(str)))
+	printFunc(box.String(title, strings.TrimSpace(str)))
 }
 
-// Boxed writes a stylized and templated message in a box to stdout
+// Boxed writes a stylized and templated message in a box to stdout using the default style config
 func Boxed(format string, a ...V) {
-	boxedCommon(String, format, a...)
+	boxedCommon(String, defaultBoxCfg, "", format, a...)
 }
 
-// BoxedErr writes a stylized and templated message in a box to stderr
+// BoxedErr writes a stylized and templated message in a box to stderr using the default style config
 func BoxedErr(format string, a ...V) {
-	boxedCommon(Err, format, a...)
+	boxedCommon(Err, defaultBoxCfg, "", format, a...)
+}
+
+// BoxedWithConfig writes a templated message in a box with customized style config to stdout
+func BoxedWithConfig(cfg box.Config, st style.Enum, title string, format string, a ...V) {
+	if st != style.None {
+		title = Sprintf(st, title)
+	}
+	// need to make sure no newlines are in the title otherwise box-cli-maker panics
+	title = strings.ReplaceAll(title, "\n", "")
+	boxedCommon(String, cfg, title, format, a...)
 }
 
 // Sprintf is used for returning the string (doesn't write anything)
@@ -369,14 +381,6 @@ func displayError(msg string, err error) {
 }
 
 func latestLogFilePath() (string, error) {
-	if len(os.Args) < 2 {
-		return "", fmt.Errorf("unable to detect command")
-	}
-	cmd := os.Args[1]
-	if cmd == "start" {
-		return localpath.LastStartLog(), nil
-	}
-
 	tmpdir := os.TempDir()
 	files, err := ioutil.ReadDir(tmpdir)
 	if err != nil {
@@ -399,16 +403,32 @@ func latestLogFilePath() (string, error) {
 	return fullPath, nil
 }
 
+func command() (string, error) {
+	if len(pflag.Args()) < 1 {
+		return "", fmt.Errorf("unable to detect command")
+	}
+
+	return pflag.Arg(0), nil
+}
+
 func displayGitHubIssueMessage() {
-	logPath, err := latestLogFilePath()
+	cmd, err := command()
 	if err != nil {
-		klog.Warningf("failed to diplay GitHub issue message: %v", err)
+		klog.Warningf("failed to get command: %v", err)
 	}
 
 	msg := Sprintf(style.Sad, "If the above advice does not help, please let us know:")
 	msg += Sprintf(style.URL, "https://github.com/kubernetes/minikube/issues/new/choose\n")
-	msg += Sprintf(style.Empty, "Please attach the following file to the GitHub issue:")
-	msg += Sprintf(style.Empty, "- {{.logPath}}", V{"logPath": logPath})
+	msg += Sprintf(style.Empty, "Please run `minikube logs --file=logs.txt` and attach logs.txt to the GitHub issue.")
+
+	if cmd != "start" {
+		logPath, err := latestLogFilePath()
+		if err != nil {
+			klog.Warningf("failed to get latest log file path: %v", err)
+		}
+		msg += Sprintf(style.Empty, "Please also attach the following file to the GitHub issue:")
+		msg += Sprintf(style.Empty, "- {{.logPath}}", V{"logPath": logPath})
+	}
 
 	BoxedErr(msg)
 }
